package com.hzya.frame.plugin.lets.dingtalk.callback.service.impl;

import cn.hutool.core.lang.Assert;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponseBody;
import com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceResponse;
import com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceResponseBody;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.dao.IArapDjzbDao;
import com.hzya.frame.plugin.lets.dingtalk.callback.service.CallBackService;
import com.hzya.frame.plugin.lets.dingtalk.dao.IDingU8cBillDao;
import com.hzya.frame.plugin.lets.dingtalk.entity.DingU8cBillEntity;
import com.hzya.frame.plugin.lets.dingtalk.utils.DingTalkUtils;
import com.hzya.frame.plugin.lets.dingtalk.utils.FileUtil;
import com.hzya.frame.plugin.lets.dingtalk.utils.PushU8CFileUtil;
import com.hzya.frame.plugin.lets.dingtalk.vo.DingCallbackCrypto;
import com.hzya.frame.plugin.lets.dingtalk.vo.FileModuleVO;
import com.hzya.frame.plugin.lets.dingtalk.vo.SysFileVO;
import com.hzya.frame.plugin.lets.entity.ArapDjzbEntity;
import com.hzya.frame.plugin.lets.entity.BdCorpEntity;
import com.hzya.frame.plugin.lets.entity.BdDeptdocEntity;
import com.hzya.frame.plugin.lets.u8cdto.*;
import com.hzya.frame.plugin.lets.util.QueryU8CEntityUtil;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.plugin.lets.util.pushData.Attribute;
import com.hzya.frame.plugin.lets.util.pushData.PushU8CByApiCode;
import com.hzya.frame.plugin.lets.util.pushData.ZTResult;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.time.LocalDate;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 钉钉回调，push_status一定是Y
 */
@Service("callBackService")
public class CallBackServiceImpl implements CallBackService {

    protected static Logger logger = LogManager.getLogger(CallBackServiceImpl.class);
    public String dingId = "";

    @Autowired
    private IDingU8cBillDao dingU8cBillDao;
    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;

    @Value("${DING.CorpId}")
    private String CorpId;
    @Value("${DING.APIToken}")
    private String APIToken;

    @Value("${DING.APPKEY}")
    private String APPKEY;
    @Value("${DING.AppSecret}")
    private String AppSecret;

    @Value("${DING.call_back_token}")
    private String token;
    @Value("${DING.call_back_aes_key}")
    private String aes_key;
    @Value("${DING.call_back_appkey}")
    private String corpId;

    @Override
    public Map<String, String> saveDingTalkCallBackRes(String msg_signature, String timeStamp, String nonce, JSONObject json) {
        logger.info("=============================================================================================================================钉钉回调执行快开始》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》》");
        try {

            // 1. 从http请求中获取加解密参数
            // 2. 使用加解密类型
            // 2、调用订阅事件接口订阅的事件为企业级事件推送，此时OWNER_KEY为：开发者后台应用的Client ID（原企业内部应用 appKey )
            //token、ase_key均为钉钉开放平台后台管理，应用设置的事件订阅,http推送中设置的参数。
            DingCallbackCrypto callbackCrypto = new DingCallbackCrypto(token, aes_key, corpId);
            String encryptMsg = json.getString("encrypt");
            String decryptMsg = callbackCrypto.getDecryptMsg(msg_signature, timeStamp, nonce, encryptMsg);

            // 3. 反序列化回调事件json数据
            JSONObject eventJson = JSON.parseObject(decryptMsg);

            String eventType = eventJson.getString("EventType");

            //防止钉钉重复回调
            if(eventJson.getString("processInstanceId")!=null) {
                if (!eventJson.getString("processInstanceId").equals(dingId)) {
                    dingId = eventJson.getString("processInstanceId");

                    // 4. 根据EventType分类处理
                    if (eventJson.getString("processCode") != null) {
                        switch (eventJson.getString("processCode")) {
                            case "PROC-EC6B5BD2-DE3C-456E-9591-6F7B377E39E4":
                                logger.info("采购付款申请（新） -> PROC-EC6B5BD2-DE3C-456E-9591-6F7B377E39E4");
                                outSourceBill(eventJson);//采购付款申请（新）
                                break;
                            case "PROC-285CCF2B-524F-4055-BE62-FC31F490C654":
                                logger.info("采购付款申请（店群专用） -> PROC-285CCF2B-524F-4055-BE62-FC31F490C654");
                                agencyBill(eventJson);//采购付款申请（店群专用）
                                break;
                            case "PROC-AC33FC85-FB44-49FC-B926-1D966AE3BBD2":
                                logger.info("采购付款申请（新） -> PROC-AC33FC85-FB44-49FC-B926-1D966AE3BBD2");
                                transferBill(eventJson);//特殊业务处理（新）
                                break;
                        }
                    }
                }
            }

            // 5. 返回success的加密数据
            Map<String, String> successMap = callbackCrypto.getEncryptedMap("success");
            logger.info("=============================================================================================================================钉钉回调执行结束《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《《");
            return successMap;
        } catch (Exception e) {
            logger.error("execute", e);
            Assert.state(false, "excute:{}", e);
        }
        return null;
    }

    /**
     * 获取钉钉应用token
     */
    public String getDingTalkToken() throws Exception {
        GetAccessTokenResponseBody tokenBody = DingTalkUtils.getTokenBody(APPKEY, AppSecret);
        return tokenBody.getAccessToken();
    }

    /**
     * 根据钉钉userId获取手机号
     *
     * @return
     */
    public String getMobileByUserId(String token, String userId) {
        OapiV2UserGetResponse user = DingTalkUtils.getUserById(token, userId);
        String mobile = user.getResult().getMobile();
        return mobile;
    }

    /**
     * 委外采购（付款单）->采购付款申请(新)
     * 回调，审核付款单
     */
    public void outSourceBill(JSONObject eventJson) throws Exception {
        try {
            //获取token
            String token = getDingTalkToken();
            //实例ID
            String processInstanceId = eventJson.getString("processInstanceId");
            DingU8cBillEntity dingU8cBillEntity = queryU8CDing(processInstanceId);
            if (dingU8cBillEntity == null) {
                Assert.state(false, "委外采购（付款单）->采购付款申请(新)回调：未查询到相关审批实例，实例id：{}", processInstanceId);
            }

            if("Y".equals(dingU8cBillEntity.getApprove())){
                return;
            }

            //查询u8c单据信息
            ArapDjzbEntity arapDjzbEntity = queryArapDjzbEntity(dingU8cBillEntity.getU8c_pk_corp(), dingU8cBillEntity.getU8c_bill_code());
            if(arapDjzbEntity==null){
                Assert.state(false,"根据公司编码:{},单据号:{},未查询到U8C付款单。",dingU8cBillEntity.getU8c_code_corp(),dingU8cBillEntity.getU8c_bill_code());
            }

            //djzt单据状态 1保存 3完成
            if("3".equals(arapDjzbEntity.getDjzt())){
                return;
            }

            //spzt审批状态 null未审批 1审批
            if("1".equals(arapDjzbEntity.getSpzt())){
                return;
            }

            //查询实例的具体信息
            GetProcessInstanceResponse processInstanceDetails = DingTalkUtils.getProcessInstanceDetails(token, processInstanceId);
            GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResult processInstanceResponseBodyResult = processInstanceDetails.getBody().getResult();
            //审批状态,该回调事件只监听，审批完成的
            if (processInstanceResponseBodyResult.getStatus().equals("RUNNING")) {//审批中
                logger.info("采购付款申请(新)--->审批中");
            } else if (processInstanceResponseBodyResult.getStatus().equals("TERMINATED")) {//已撤销
                logger.info("采购付款申请(新)--->已撤销");
            } else if (processInstanceResponseBodyResult.getStatus().equals("COMPLETED")) {//审批完成
                logger.info("采购付款申请(新)--->审批完成");
            }

            String status = null;//审批状态。RUNNING：审批中 TERMINATED：已撤销 COMPLETED：审批完成
            String result = null;//审批结果。agree：同意 refuse：拒绝
            String userId = null;//任务处理人。
//            String mobile = null;//处理人手机号

            status = processInstanceResponseBodyResult.getStatus();
            if ("RUNNING".equals(status) || "TERMINATED".equals(status)) {
                //审批中、已撤销不处理
                return;
            }

            //任务列表,文档地址：https://open-dev.dingtalk.com/apiExplorer?spm=ding_open_doc.document.0.0.33293c33eWKtYM#/?devType=org&api=workflow_1.0%23GetProcessInstance
            List<GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultTasks> tasks = processInstanceResponseBodyResult.getTasks();
            for (GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultTasks task : tasks) {
                userId=task.getUserId();
            }
//            mobile = getMobileByUserId(token, userId);

            result = processInstanceResponseBodyResult.getResult();
            if ("agree".equals(result)) {
                //同意
                //审批u8c单据
                approveArapDjzb(dingU8cBillEntity);
            }else if ("refuse".equals(result)) {
                //拒绝
                dingU8cBillEntity.setApprove("N");
                dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
                //后续林姐姐给接口，拒绝之后清空process_id、approve：N
                refuse(dingU8cBillEntity);
            }
        } catch (Exception e) {
            logger.error("回调报错：" + e);
            LocalDate now = LocalDate.now();

            //做一个钉钉的日志
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
            integrationTaskLivingDetailsEntity.setRootAppNewData("委外采购（付款单）->采购付款申请(新)回调报错");
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(e.getMessage());
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(now.toString());
            integrationTaskLivingDetailsEntity.setRootAppPk(eventJson.getString("processInstanceId"));
            integrationTaskLivingDetailsEntity.setRootAppBill(eventJson.getString("processInstanceId"));
            integrationTaskLivingDetailsEntity.setPluginId("dingtalk.DingCallBackServicePluginInitializer");
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
        }

    }

    /**
     * 代理采购（付款单）->采购付款申请(店群专用)
     * 回调，审核付款单
     */
    public void agencyBill(JSONObject eventJson) throws Exception {
        try {
            //获取token
            String token = getDingTalkToken();
            //实例ID
            String processInstanceId = eventJson.getString("processInstanceId");
            DingU8cBillEntity dingU8cBillEntity = queryU8CDing(processInstanceId);
            if (dingU8cBillEntity == null) {
                Assert.state(false, "代理采购（付款单）->采购付款申请(店群专用)回调：未查询到相关审批实例，实例id：{}", processInstanceId);
            }


            if("Y".equals(dingU8cBillEntity.getApprove())){
                return;
            }

            //查询u8c单据信息
            ArapDjzbEntity arapDjzbEntity = queryArapDjzbEntity(dingU8cBillEntity.getU8c_pk_corp(), dingU8cBillEntity.getU8c_bill_code());
            if(arapDjzbEntity==null){
                Assert.state(false,"根据公司编码:{},单据号:{},未查询到U8C付款单。",dingU8cBillEntity.getU8c_code_corp(),dingU8cBillEntity.getU8c_bill_code());
            }

            //djzt单据状态 1保存 3完成
            if("3".equals(arapDjzbEntity.getDjzt())){
                return;
            }

            //spzt审批状态 null未审批 1审批
            if("1".equals(arapDjzbEntity.getSpzt())){
                return;
            }

            //查询实例的具体信息
            GetProcessInstanceResponse processInstanceDetails = DingTalkUtils.getProcessInstanceDetails(token, processInstanceId);
            GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResult processInstanceResponseBodyResult = processInstanceDetails.getBody().getResult();
            //审批状态,该回调事件只监听，审批完成的
            if (processInstanceResponseBodyResult.getStatus().equals("RUNNING")) {//审批中
                logger.info("采购付款申请(店群专用)--->审批中");
            } else if (processInstanceResponseBodyResult.getStatus().equals("TERMINATED")) {//已撤销
                logger.info("采购付款申请(店群专用)--->已撤销");
            } else if (processInstanceResponseBodyResult.getStatus().equals("COMPLETED")) {//审批完成
                logger.info("采购付款申请(店群专用)--->审批完成");
            }

            String status = null;//审批状态。RUNNING：审批中 TERMINATED：已撤销 COMPLETED：审批完成
            String result = null;//审批结果。agree：同意 refuse：拒绝
            String userId = null;//任务处理人。
//            String mobile = null;//处理人手机号

            status = processInstanceResponseBodyResult.getStatus();
            if ("RUNNING".equals(status) || "TERMINATED".equals(status)) {
                //审批中、已撤销不处理
                return;
            }

            //任务列表,文档地址：https://open-dev.dingtalk.com/apiExplorer?spm=ding_open_doc.document.0.0.33293c33eWKtYM#/?devType=org&api=workflow_1.0%23GetProcessInstance
            List<GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultTasks> tasks = processInstanceResponseBodyResult.getTasks();
            for (GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultTasks task : tasks) {
                userId=task.getUserId();
            }
//            mobile = getMobileByUserId(token, userId);

            result = processInstanceResponseBodyResult.getResult();
            if ("agree".equals(result)) {
                //同意
                //审批u8c单据
                approveArapDjzb(dingU8cBillEntity);
            }else if ("refuse".equals(result)) {
                //拒绝
                dingU8cBillEntity.setApprove("N");
                dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
                //后续林姐姐给接口，拒绝之后清空process_id、approve：N
                refuse(dingU8cBillEntity);
            }
        } catch (Exception e) {
            logger.error("回调报错：" + e);
            LocalDate now = LocalDate.now();

            //做一个钉钉的日志
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
            integrationTaskLivingDetailsEntity.setRootAppNewData("代理采购（付款单）->采购付款申请(店群专用)回调报错");
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(e.getMessage());
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(now.toString());
            integrationTaskLivingDetailsEntity.setRootAppPk(eventJson.getString("processInstanceId"));
            integrationTaskLivingDetailsEntity.setRootAppBill(eventJson.getString("processInstanceId"));
            integrationTaskLivingDetailsEntity.setPluginId("dingtalk.DingCallBackServicePluginInitializer");
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);

        }
    }

    /**
     * 根据ding_talk_process_id查询
     */
    public DingU8cBillEntity queryU8CDing(String ding_talk_process_id) {
        DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity();
        dingU8cBillEntity.setDing_talk_process_id(ding_talk_process_id);
        dingU8cBillEntity.setSts("Y");
        List<DingU8cBillEntity> query = dingU8cBillDao.query(dingU8cBillEntity);
        if (query.size() == 0) {
            return null;
        } else {
            return query.get(0);
        }
    }

    /**
     * 调拨单内部交易（付款单）->特殊业务处理(新)
     * 回调，审核付款单
     * 两种业务场景
     * 1、付款单审核
     * 2、付款单新增
     */
    public void transferBill(JSONObject eventJson) throws Exception {
        try {
            //获取token
            String token = getDingTalkToken();
            //实例ID
            String processInstanceId = eventJson.getString("processInstanceId");
            DingU8cBillEntity dingU8cBillEntity = queryU8CDing(processInstanceId);
            if (dingU8cBillEntity == null) {
                //新增u8c付款单、保存日志、上传附件
                addArapBill(processInstanceId, token);
            } else {

                if("Y".equals(dingU8cBillEntity.getApprove())){
                    return;
                }

                //查询u8c单据信息
                ArapDjzbEntity arapDjzbEntity = queryArapDjzbEntity(dingU8cBillEntity.getU8c_pk_corp(), dingU8cBillEntity.getU8c_bill_code());
                if(arapDjzbEntity==null){
                    Assert.state(false,"根据公司编码:{},单据号:{},未查询到U8C付款单。",dingU8cBillEntity.getU8c_code_corp(),dingU8cBillEntity.getU8c_bill_code());
                }

                //djzt单据状态 1保存 3完成
                if("3".equals(arapDjzbEntity.getDjzt())){
                    return;
                }

                //spzt审批状态 null未审批 1审批
                if("1".equals(arapDjzbEntity.getSpzt())){
                    return;
                }



                //查询实例的具体信息
                GetProcessInstanceResponse processInstanceDetails = DingTalkUtils.getProcessInstanceDetails(token, processInstanceId);
                GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResult processInstanceResponseBodyResult = processInstanceDetails.getBody().getResult();
                //审批状态,该回调事件只监听，审批完成的
                if (processInstanceResponseBodyResult.getStatus().equals("RUNNING")) {//审批中
                    logger.info("特殊业务处理(新)--->审批中");
                } else if (processInstanceResponseBodyResult.getStatus().equals("TERMINATED")) {//已撤销
                    logger.info("特殊业务处理(新)--->已撤销");
                } else if (processInstanceResponseBodyResult.getStatus().equals("COMPLETED")) {//审批完成
                    logger.info("特殊业务处理(新)--->审批完成");
                }
                //获取所有审批意见
                List<GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultOperationRecords> operationRecords = processInstanceResponseBodyResult.getOperationRecords();

                //审批意见过滤掉：START_PROCESS_INSTANCE：发起流程实例
                operationRecords = operationRecords.stream().filter(item -> !item.getType().equals("START_PROCESS_INSTANCE")).collect(Collectors.toList());
                System.out.println(operationRecords);

                String status = null;//审批状态。RUNNING：审批中 TERMINATED：已撤销 COMPLETED：审批完成
                String result = null;//审批结果。agree：同意 refuse：拒绝
                String userId = null;//任务处理人。
//                String mobile = null;//处理人手机号

                status = processInstanceResponseBodyResult.getStatus();
                if ("RUNNING".equals(status) || "TERMINATED".equals(status)) {
                    //审批中、已撤销不处理
                    return;
                }

                //任务列表,文档地址：https://open-dev.dingtalk.com/apiExplorer?spm=ding_open_doc.document.0.0.33293c33eWKtYM#/?devType=org&api=workflow_1.0%23GetProcessInstance
                List<GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultTasks> tasks = processInstanceResponseBodyResult.getTasks();
                for (GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultTasks task : tasks) {
                    userId=task.getUserId();
                }
//                mobile = getMobileByUserId(token, userId);

                result = processInstanceResponseBodyResult.getResult();
                if ("agree".equals(result)) {
                    //同意
                    //审批u8c单据
                    approveArapDjzb(dingU8cBillEntity);
                }else if ("refuse".equals(result)) {
                    //拒绝
                    dingU8cBillEntity.setApprove("N");
                    dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
                    //后续林姐姐给接口，拒绝之后清空process_id、approve：N
                    refuse(dingU8cBillEntity);
                }
            }

        } catch (Exception e) {
            logger.error("回调报错：" + e);
            LocalDate now = LocalDate.now();

            //做一个钉钉的日志
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
            integrationTaskLivingDetailsEntity.setRootAppNewData("调拨单内部交易（付款单）->特殊业务处理(新)回调报错");
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(e.getMessage());
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(now.toString());
            integrationTaskLivingDetailsEntity.setRootAppPk(eventJson.getString("processInstanceId"));
            integrationTaskLivingDetailsEntity.setRootAppBill(eventJson.getString("processInstanceId"));
            integrationTaskLivingDetailsEntity.setPluginId("dingtalk.DingCallBackServicePluginInitializer");
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);

        }
    }

    @Autowired
    private PushU8CByApiCode pushU8CByApiCode;

    /**
     * 审核付款单,并保存日志
     */
    public void approveArapDjzb(DingU8cBillEntity dingU8cBillEntity) throws Exception {
        //获取当前时间
        LocalDate now = LocalDate.now();

        Map bills = new HashMap();
        bills.put("djbh", dingU8cBillEntity.getU8c_bill_code());
        bills.put("dwbm", dingU8cBillEntity.getU8c_code_corp());
        bills.put("opinion", "同意");
        bills.put("operatorid", OverallConstant.getOverAllValue("u8cApiZdrCode"));
//        bills.put("operatorid", mobile);
        bills.put("operatedate", now.toString());
        String jsonStr = JSONUtil.toJsonStr(bills);
        jsonStr = "{\"bills\":[" + jsonStr + "]}";
        System.out.println(jsonStr);

        //推送
        String response = "";
        try {
            ZTResult ztResult = pushU8CByApiCode.pushByCode(OverallConstant.getOverAllValue("arapFkInsetCode"), jsonStr);
            //两层判断,一层中台转发,一层u8c返回
            if ("false".equals(ztResult.getFlag())) {
                //转发失败
                response = JSONUtil.toJsonStr(ztResult.getAttribute());
                Assert.state(false, "丽知：审核付款单审核失败,失败原因:{}", ztResult.getAttribute());
            }
            Attribute attribute = ztResult.getAttribute();
            boolean isSuccess = false;
            if ("success".equals(attribute.getStatus())) {
                isSuccess = true;
            }
            if (!isSuccess) {
                Assert.state(false, "推送U8C--> 丽知：审核付款单审核 失败 接口返回结果：{} ", response);
            }
            //成功
            dingU8cBillEntity.setApprove("Y");
            dingU8cBillEntity.setDing_res("同意");
            dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
        } catch (Exception e) {
            logger.error("推送U8C--> 丽知：审核付款单审核 失败" + e);
            //失败
            dingU8cBillEntity.setApprove("N");
            dingU8cBillEntity.setPush_info(e.getMessage());
            dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
        }

    }

    @Autowired
    private QueryU8CEntityUtil queryU8CEntityUtil;
    @Autowired
    private FileUtil fileUtil;
    @Autowired
    private PushU8CFileUtil pushU8CFileUtil;

    /**
     * 付款单新增
     * 1、付款单保存及审批
     * 2、附件上传
     */
    public void addArapBill(String processInstanceId, String token) throws Exception {
        //查询实例的具体信息
        GetProcessInstanceResponse processInstanceDetails = DingTalkUtils.getProcessInstanceDetails(token, processInstanceId);
        GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResult processInstanceResponseBodyResult = processInstanceDetails.getBody().getResult();
        List<GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultFormComponentValues> formComponentValues = processInstanceResponseBodyResult.getFormComponentValues();

        //需要的信息：公司、部门、项目所属类目、费用所属项目、申请理由、申请金额、备注、附件
        String corpName = "";
        String corpCode = "";
        String deptName = "";
        String deptCode = "";
        String xmName = "";
        String xmCode = "";
        String fyName = "";
        String fyCode = "";
        String ly = "";
        String je = "";
        String bz = "";
        String fjJson = "";
        for (GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResultFormComponentValues formComponentValue : formComponentValues) {
            if ("费用所属公司".equals(formComponentValue.getName())) {
                corpName = formComponentValue.getValue();
            }
            if ("费用所属部门".equals(formComponentValue.getName())) {
                deptName = formComponentValue.getValue();
            }
            if ("项目所属类目".equals(formComponentValue.getName())) {
                xmName = formComponentValue.getValue();
            }
            if ("费用所属项目".equals(formComponentValue.getName())) {
                fyName = formComponentValue.getValue();
            }
            if ("申请理由".equals(formComponentValue.getName())) {
                ly = formComponentValue.getValue();
            }
            if ("申请金额（元）".equals(formComponentValue.getName())) {
                je = formComponentValue.getValue();
            }
            if ("备注".equals(formComponentValue.getName())) {
                bz = formComponentValue.getValue();
            }
            if ("DDAttachment".equals(formComponentValue.getComponentType())) {
                fjJson = formComponentValue.getValue();
            }
        }

        String mapStr = "";
        String response = "";

        BdCorpEntity bdCorpEntity_w=null;

        try {
            ArapDjzbVO billvo = new ArapDjzbVO();
            ArapDjzbVO.Parentt parentvo = new ArapDjzbVO.Parentt();
            //djrq 单据日期
            LocalDate now = LocalDate.now();
            parentvo.setDjrq(now.toString());

            //dwbm 公司
            BdCorpEntity bdCorpEntity = queryU8CEntityUtil.queryBdCorpByUnitName(corpName);
            if (bdCorpEntity == null) {
                Assert.state(false, "钉钉回调->付款单新增：根据表单【费用所属公司】：{}，未查询到u8c对应公司，请检查名称。", corpName);
            }
            bdCorpEntity_w=bdCorpEntity;
            parentvo.setDwbm(bdCorpEntity.getUnitcode());

            //djlxbm单据类型编码 采购付款单 D3
            parentvo.setDjlxbm("D3");

            //wldx 往来对象标识(0 客户 1供应商 2部门 3业务员 )
            parentvo.setWldx("2");

            //deptid 部门
            BdDeptdocEntity bdDeptdocEntity = queryU8CEntityUtil.queryBdDeptDocByPkCorpAndDeptName(bdCorpEntity.getPkCorp(), deptName);
            if (bdDeptdocEntity == null) {
                Assert.state(false, "钉钉回调->付款单新增：根据表单【费用所属部门】：{}。在公司：{}，下未查询到u8c对应部门，请检查部门名称。", corpName, deptName);
            }
            parentvo.setDeptid(bdDeptdocEntity.getDeptcode());

            //lrr 录入人
            parentvo.setLrr(OverallConstant.getOverAllValue("u8cApiZdrCode"));
            //shr 审核人
            parentvo.setShr(OverallConstant.getOverAllValue("u8cApiZdrCode"));
            //shrq 审核日期
            parentvo.setShrq(now.toString());

            List<ArapDjzbVO.Childrenn> children = new ArrayList<>();
            ArapDjzbVO.Childrenn childrenvo = new ArapDjzbVO.Childrenn();
            //jfybje 借方原币金额
            childrenvo.setJfybje(je);
            children.add(childrenvo);
            billvo.setParentvo(parentvo);
            billvo.setChildren(children);
            mapStr = "{\"billvo\":[" + JSONUtil.toJsonStr(billvo) + "]}";
            //推送u8c
            ZTResult ztResult = pushU8CByApiCode.pushByCode(OverallConstant.getOverAllValue("arapFkApproveCode"), mapStr);
            if ("false".equals(ztResult.getFlag())) {
                //转发失败
                response = JSONUtil.toJsonStr(ztResult.getAttribute());
                Assert.state(false, "付款单新增.中台转发失败,失败原因:{}", ztResult.getAttribute());
            }
            Attribute attribute = ztResult.getAttribute();
            boolean isSuccess = false;
            String vbillcode = "";
            String vbillId = "";
            if ("success".equals(attribute.getStatus())) {
                isSuccess = true;
                String data = attribute.getData();
                ArapDjzbVO arapDjzbVO = resultDataHandle(data);
                vbillcode = arapDjzbVO.getParentvo().getDjbh();
                vbillId = arapDjzbVO.getParentvo().getVouchid();
            }
            if (!isSuccess) {
                Assert.state(false, "推送U8C--> 付款单新增 失败 接口返回结果：{} 接口入参:{}", attribute.getErrormsg(), mapStr);
            }

            //成功
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity();
            dingU8cBillEntity.setU8c_pk_corp(bdCorpEntity.getPkCorp());//公司主键
            dingU8cBillEntity.setU8c_code_corp(bdCorpEntity.getUnitcode());//公司编码
            dingU8cBillEntity.setU8c_name_corp(corpName);//公司名称
            dingU8cBillEntity.setU8c_bill_type("D3");//单据类型
            dingU8cBillEntity.setU8c_bill_code(vbillcode);//单据号
            dingU8cBillEntity.setU8c_bill_id(vbillId);//单据主键
            dingU8cBillEntity.setPush_status("Y");//推送成功
            dingU8cBillEntity.setPush_info("success");//推送成功
            dingU8cBillEntity.setDing_talk_process_id(processInstanceId);//钉钉审批实例id
            dingU8cBillEntity.setCreate_time(new Date());//创建时间
            dingU8cBillEntity.setApprove("Y");//Y审核 N不审核
            dingU8cBillEntity.setInitiate("D");//D钉钉发起，U u8c发起
            dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);

            //上传附件,获取附件的表单，转base64，上传u8c
            List<FileModuleVO> fileModuleVOList = JSONUtil.toList(fjJson, FileModuleVO.class);
            List<SysFileVO> sysFileVOS = fileUtil.DingFileTransBase64(fileModuleVOList, token, processInstanceId);

            for (SysFileVO sysFileVO : sysFileVOS) {
                //上传u8c单据
                pushU8CFileUtil.pushFile(vbillId, sysFileVO);
            }
        } catch (Exception e) {
            logger.error(e);
            //失败
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity();
            dingU8cBillEntity.setU8c_code_corp(bdCorpEntity_w==null?null:bdCorpEntity_w.getPkCorp());//公司主键
            dingU8cBillEntity.setU8c_code_corp(bdCorpEntity_w==null?null:bdCorpEntity_w.getUnitcode());//公司编码
            dingU8cBillEntity.setU8c_name_corp(corpName);//公司编码
            dingU8cBillEntity.setU8c_bill_type("D3");//单据类型
            dingU8cBillEntity.setU8c_bill_code(null);//单据号
            dingU8cBillEntity.setU8c_bill_id(null);//单据主键
            dingU8cBillEntity.setPush_status("N");//推送失败
            dingU8cBillEntity.setPush_info(e.getMessage());//失败消息
            dingU8cBillEntity.setDing_talk_process_id(processInstanceId);//钉钉审批实例id
            dingU8cBillEntity.setCreate_time(new Date());//创建时间
            dingU8cBillEntity.setApprove(null);//Y审核 N不审核
            dingU8cBillEntity.setInitiate("D");//D钉钉发起，U u8c发起
            dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);


            LocalDate now = LocalDate.now();

            //做一个钉钉的日志
            IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = new IntegrationTaskLivingDetailsEntity();
            integrationTaskLivingDetailsEntity.setNewState(ProfilesActiveConstant.LOG_STATUS_N);
            integrationTaskLivingDetailsEntity.setRootAppNewData("付款单新增回调报错");
            integrationTaskLivingDetailsEntity.setNewTransmitInfo(e.getMessage());
            integrationTaskLivingDetailsEntity.setNewPushDate(new Date());
            integrationTaskLivingDetailsEntity.setBusinessDate(now.toString());
            integrationTaskLivingDetailsEntity.setRootAppPk(processInstanceId);
            integrationTaskLivingDetailsEntity.setRootAppBill(processInstanceId);
            integrationTaskLivingDetailsEntity.setPluginId("dingtalk.DingCallBackServicePluginInitializer");
            saveOrUpdateBusinessLogUtil.saveOrUpdate(integrationTaskLivingDetailsEntity);
        }
    }

    private ArapDjzbVO resultDataHandle(String resultData) {
        try {
            if (resultData != null && !"".equals(resultData)) {
                if (resultData.contains("[")) {
                    resultData = resultData.substring(1, resultData.length() - 1);
                }
                return JSON.parseObject(resultData, ArapDjzbVO.class);
            }
        } catch (Exception e) {
            e.printStackTrace();
            logger.error("解析返回参数失败的错误", e);
            //如果解析失败，记录原因，但是不能影响结果的记录
        }
        return null;
    }

    /**
     * 钉钉拒绝审批，http请求
     */
    public void refuse(DingU8cBillEntity dingU8cBillEntity) throws IOException {
        // 服务器访问地址及端口,例如 http://ip:port
        String serviceUrl = OverallConstant.getOverAllValue("u8cApiUrl");
        // 接口地址
        String serviceName = "/u8cloud/api/dev/define";

        //请求头
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("usercode", OverallConstant.getOverAllValue("u8cApiUsercodeCode")); // 用户
        map.put("password", OverallConstant.getOverAllValue("u8cApiPasswordCode")); // 密码，需要 MD5 加密后录入
        map.put("system", OverallConstant.getOverAllValue("u8cApiSystemCode")); // 系统编码
        map.put("custdevpclass", "u8c.api.dev.dingTalk.RefuseApprove"); //方法路径

        //请求体
        Map<String, String> fileMap = new HashMap<>();
        fileMap.put("instanceId", dingU8cBillEntity.getDing_talk_process_id());
//        fileMap.put("userCode", mobile);//审批人
        fileMap.put("userCode", OverallConstant.getOverAllValue("u8cApiZdrCode"));//审批人
        fileMap.put("ckeckResult", "R");
        fileMap.put("ckeckNote", "无");
        String jsonFileMap = JSONUtil.toJsonStr(fileMap);


        //请求
        HttpClient httpClient = new HttpClient();
        PostMethod httpPost = new PostMethod(serviceUrl + serviceName);
        httpPost.setRequestHeader("content-type", "application/json;charset=utf-8");
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            httpPost.setRequestHeader(entry.getKey(), entry.getValue()
                    .toString());
        }

        RequestEntity entity = new StringRequestEntity(jsonFileMap, "application/json", "UTF-8");
        httpPost.setRequestEntity(entity);
        httpClient.executeMethod(httpPost);
        String responseBodyAsString = httpPost.getResponseBodyAsString();
        System.out.println(responseBodyAsString);
    }

    @Autowired
    private IArapDjzbDao arapDjzbDao;
    /**
     * 查询u8c付款单主表
     */
    public ArapDjzbEntity queryArapDjzbEntity(String u8cPkCorp, String u8cBillCode){
        ArapDjzbEntity arapDjzbEntity = new ArapDjzbEntity();
        arapDjzbEntity.setDr("0");
        arapDjzbEntity.setDwbm(u8cPkCorp);
        arapDjzbEntity.setDjbh(u8cBillCode);
        arapDjzbEntity.setDataSourceCode(ProfilesActiveConstant.LETS_DATE_SOURCE);
        List<ArapDjzbEntity> query = arapDjzbDao.query(arapDjzbEntity);
        if(query.size()==0){
            return null;
        }else {
            return query.get(0);
        }
    }

}
