package com.hzya.frame.plugin.lets.plugin.dingtalk;

import cn.hutool.core.lang.Assert;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.aliyun.dingtalkoauth2_1_0.models.GetAccessTokenResponseBody;
import com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceResponse;
import com.aliyun.dingtalkworkflow_1_0.models.GetProcessInstanceResponseBody;
import com.dingtalk.api.response.OapiV2UserGetResponse;
import com.hzya.frame.base.PluginBaseEntity;
import com.hzya.frame.plugin.lets.constant.OverallConstant;
import com.hzya.frame.plugin.lets.constant.ProfilesActiveConstant;
import com.hzya.frame.plugin.lets.dao.IArapDjzbDao;
import com.hzya.frame.plugin.lets.dingtalk.dao.IDingU8cBillDao;
import com.hzya.frame.plugin.lets.dingtalk.entity.DingU8cBillEntity;
import com.hzya.frame.plugin.lets.dingtalk.utils.DingTalkUtils;
import com.hzya.frame.plugin.lets.entity.ArapDjzbEntity;
import com.hzya.frame.plugin.lets.u8cdto.ArapDjzbVO;
import com.hzya.frame.plugin.lets.util.SaveOrUpdateBusinessLogUtil;
import com.hzya.frame.plugin.lets.util.pushData.Attribute;
import com.hzya.frame.plugin.lets.util.pushData.PushU8CByApiCode;
import com.hzya.frame.plugin.lets.util.pushData.ZTResult;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.dao.IIntegrationTaskLivingDetailsDao;
import com.hzya.frame.sysnew.integtationTaskLivingDetails.entity.IntegrationTaskLivingDetailsEntity;
import com.hzya.frame.web.entity.BaseResult;
import com.hzya.frame.web.entity.JsonResultEntity;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.time.LocalDate;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.locks.ReentrantLock;
import java.util.stream.Collectors;

/**
 * 为避免回调超时，每天定时检查所有未审批完成的审批实例。
 * 只检测u8c成功推送钉钉的，不检测钉钉自己发起的审批实例
 */
@Service("DingCallBackService")
public class DingCallBackPluginInitializer extends PluginBaseEntity {
    Logger logger = LoggerFactory.getLogger(DingCallBackPluginInitializer.class);

    private static final ReentrantLock LOCK = new ReentrantLock(true);
    @Value("${DING.APPKEY}")
    private String APPKEY;
    @Value("${DING.AppSecret}")
    private String AppSecret;

    @Autowired
    private IIntegrationTaskLivingDetailsDao iIntegrationTaskLivingDetailsDao;

    @Autowired
    private SaveOrUpdateBusinessLogUtil saveOrUpdateBusinessLogUtil;
    @Autowired
    private IDingU8cBillDao dingU8cBillDao;

    @Override
    public void initialize() {
        logger.info(getPluginLabel() + "執行初始化方法initialize()");
    }

    @Override
    public void destroy() {
        logger.info(getPluginLabel() + "執行銷毀方法destroy()");
    }

    @Override
    public String getPluginId() {
        return "dingtalk.DingCallBackPluginInitializer";
    }

    @Override
    public String getPluginName() {
        return "丽知：钉钉审批实例自动检查";
    }

    @Override
    public String getPluginLabel() {
        return "丽知：钉钉审批实例自动检查";
    }

    @Override
    public String getPluginType() {
        return "1";
    }

    @Override
    public JsonResultEntity executeBusiness(JSONObject requestJson) throws Exception {
        try {
            logger.info("调用：" + getPluginName() + "-插件");
            String prod = "prod";
            String param = String.valueOf(requestJson.get("param"));
            if (requestJson != null && ProfilesActiveConstant.TYPE_VBILLCODE.equals(requestJson.get("type")) && param != null && !"".equals(param)) {
                //按单号
                start(param);
            } else if (ProfilesActiveConstant.TYPE_DETAIL_ERROR.equals(requestJson.get("type"))) {
                String integrationTaskLivingDetails = (String) requestJson.get("integration_task_living_details_id");
                IntegrationTaskLivingDetailsEntity integrationTaskLivingDetailsEntity = saveOrUpdateBusinessLogUtil.queryIntegrationTaskLivingDetailsEntity(integrationTaskLivingDetails);
                start(integrationTaskLivingDetailsEntity.getRootAppPk());
            } else if (ProfilesActiveConstant.LETS_PROFILES_ACTIVE.equals(prod)) {
                //默认
                start();
            }
        } catch (Exception e) {
            logger.error("executeBusiness方法抛出异常", e);
        }
        return BaseResult.getSuccessMessageEntity("插件执行成功");
    }


    /**
     * 检测所有插件
     * 只检测u8c成功推送钉钉的 ，不检测钉钉自己发起的审批实例
     */
    public void start() {
        try {
            LocalDate now = LocalDate.now();
            // 减去3个月
            LocalDate nowBeforeThreeMonths = now.minus(3, ChronoUnit.MONTHS);

            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity();
            dingU8cBillEntity.setStart_time(nowBeforeThreeMonths+" 00:00:00");
            dingU8cBillEntity.setEnd_time(now+" 23:59:59");
            dingU8cBillEntity.setSts("Y");
            dingU8cBillEntity.setInitiate("U");
            dingU8cBillEntity.setPush_status("Y");
            List<DingU8cBillEntity> query = dingU8cBillDao.query(dingU8cBillEntity);
            if (query == null || query.size() == 0) {
                return;
            }
            //过滤同意 且 实例id不为null
            List<DingU8cBillEntity> filterData = filterPass(query);
            if (filterData == null || filterData.size() == 0) {
                return;
            }
            implement(filterData);
        } catch (Exception e) {
            logger.error("丽知：钉钉审批实例自动检查start()报错，报错信息：{}", e);
        }
    }

    /**
     * 检测是定插件
     *
     * @param processInstanceId
     */
    public void start(String processInstanceId) {
        try {
            DingU8cBillEntity dingU8cBillEntity = new DingU8cBillEntity();
            dingU8cBillEntity.setDing_talk_process_id(processInstanceId);
            dingU8cBillEntity.setSts("Y");
            dingU8cBillEntity.setInitiate("U");
            List<DingU8cBillEntity> query = dingU8cBillDao.query(dingU8cBillEntity);
            if (query == null || query.size() == 0) {
                Assert.state(false,"根据审批实例id：{}，无法查询到推送记录",processInstanceId);
            }
            //过滤同意 且 实例id不为null
            List<DingU8cBillEntity> filterData = filterPass(query);
            if (filterData == null || filterData.size() == 0) {
                return;
            }
            implement(filterData);
        } catch (Exception e) {
            logger.error("丽知：钉钉审批实例自动检查start(String processInstanceId)报错，报错信息：{}", e);
        }

    }

    /**
     * 过滤审核通过的
     *
     * @param query
     * @return
     */
    public List<DingU8cBillEntity> filterPass(List<DingU8cBillEntity> query) {
        List<DingU8cBillEntity> filter = new ArrayList<>();
        for (DingU8cBillEntity dingU8cBillEntity : query) {
            if ("Y".equals(dingU8cBillEntity.getApprove()) || (dingU8cBillEntity.getDing_talk_process_id() == null || "".equals(dingU8cBillEntity.getDing_talk_process_id().trim()))) {
                continue;
            }
            if(dingU8cBillEntity.getApprove()==null){
                filter.add(dingU8cBillEntity);
            }
        }
        return filter;

    }

    /**
     * 查询表单，看返回接口。
     * push_status:Y approve:(null)
     * @param filterData
     */
    public void implement(List<DingU8cBillEntity> filterData) throws Exception {
        //获取token
        String token = getDingTalkToken();
        for (DingU8cBillEntity dingU8cBillEntity : filterData) {

            try {
                //查询实例的具体信息
                GetProcessInstanceResponse processInstanceDetails = DingTalkUtils.getProcessInstanceDetails(token, dingU8cBillEntity.getDing_talk_process_id());
                GetProcessInstanceResponseBody.GetProcessInstanceResponseBodyResult processInstanceResponseBodyResult = processInstanceDetails.getBody().getResult();

                String status = null;//审批状态。RUNNING：审批中 TERMINATED：已撤销 COMPLETED：审批完成
                String result = null;//审批结果。agree：同意 refuse：拒绝
                String userId = null;//任务处理人。
                String mobile = null;//处理人手机号

                status = processInstanceResponseBodyResult.getStatus();
                if ("RUNNING".equals(status) || "TERMINATED".equals(status)) {
                    //审批中、已撤销不处理
                    return;
                }

                //查询u8c的单据状态
                String u8cPkCorp = dingU8cBillEntity.getU8c_pk_corp();
                String u8cBillCode = dingU8cBillEntity.getU8c_bill_code();
                ArapDjzbEntity arapDjzbEntity = queryArapDjzbEntity(u8cPkCorp, u8cBillCode);
                if(arapDjzbEntity==null){
                    Assert.state(false,"根据公司主键：{}，单据号：{}，未查询到u8c付款单，请检查。",u8cPkCorp,u8cBillCode);
                }

                //djzt单据状态 1保存 3完成
                if("3".equals(arapDjzbEntity.getDjzt())){
                    continue;
                }
                //spzt审批状态 null未审批 1审批
                if("1".equals(arapDjzbEntity.getSpzt())){
                    continue;
                }

                result = processInstanceResponseBodyResult.getResult();
                if ("agree".equals(result)) {
                    //同意
                    //审批u8c单据
                    approveArapDjzb(dingU8cBillEntity);
                }else if ("refuse".equals(result)) {
                    //拒绝
                    dingU8cBillEntity.setApprove("N");
                    dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
                    //后续林姐姐给接口，拒绝之后清空process_id、approve：N
                    refuse(dingU8cBillEntity);
                }


            } catch (Exception e) {
                logger.error("丽知：钉钉审批实例自动检查implement(List<String> processInstanceIds)报错，报错信息：{}", e);

            }
        }
    }


    @Autowired
    private PushU8CByApiCode pushU8CByApiCode;
    /**
     * 审核付款单
     */
    public void approveArapDjzb(DingU8cBillEntity dingU8cBillEntity) throws Exception {

        LocalDate now = LocalDate.now();

        Map bills = new HashMap();
        bills.put("djbh", dingU8cBillEntity.getU8c_bill_code());
        bills.put("dwbm", dingU8cBillEntity.getU8c_code_corp());
        bills.put("opinion", "同意");
//        bills.put("operatorid", OverallConstant.getOverAllValue("u8cApiZdrCode"));
        bills.put("operatorid", "tbadmin");
        bills.put("operatedate", now.toString());
        String jsonStr = JSONUtil.toJsonStr(bills);
        jsonStr = "{\"bills\":[" + jsonStr + "]}";
        System.out.println(jsonStr);

        //推送
        String response = "";
        try {
            ZTResult ztResult = pushU8CByApiCode.pushByCode(OverallConstant.getOverAllValue("arapFkApproveCode"), jsonStr);
            //两层判断,一层中台转发,一层u8c返回
            if ("false".equals(ztResult.getFlag())) {
                //转发失败
                response = JSONUtil.toJsonStr(ztResult.getAttribute());
                Assert.state(false, "丽知：审核付款单审核失败,失败原因:{}", ztResult.getAttribute());
            }
            Attribute attribute = ztResult.getAttribute();
            boolean isSuccess = false;
            if ("success".equals(attribute.getStatus())) {
                isSuccess = true;
            }
            if (!isSuccess) {
                Assert.state(false, "推送U8C--> 丽知：审核付款单审核 失败 接口返回结果：{} ", response);
            }
            //成功
            dingU8cBillEntity.setApprove("Y");
            dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
        } catch (Exception e) {
            logger.error("推送U8C--> 丽知：审核付款单审核 失败" + e);
            //失败
            dingU8cBillEntity.setApprove("N");
            dingU8cBillEntity.setPush_info(e.getMessage());
            dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
        }

    }

    /**
     * 钉钉拒绝审批，http请求
     */
    public void refuse(DingU8cBillEntity dingU8cBillEntity) throws IOException {
        // 服务器访问地址及端口,例如 http://ip:port
        String serviceUrl = OverallConstant.getOverAllValue("u8cApiUrl");
        // 接口地址
        String serviceName = "/u8cloud/api/dev/define";

        //请求头
        Map<String, Object> map = new HashMap<String, Object>();
        map.put("usercode", OverallConstant.getOverAllValue("u8cApiUsercodeCode")); // 用户
        map.put("password", OverallConstant.getOverAllValue("u8cApiPasswordCode")); // 密码，需要 MD5 加密后录入
        map.put("system", OverallConstant.getOverAllValue("u8cApiSystemCode")); // 系统编码
        map.put("custdevpclass", "u8c.api.dev.dingTalk.RefuseApprove"); //方法路径

        //请求体
        Map<String, String> fileMap = new HashMap<>();
        fileMap.put("instanceId", dingU8cBillEntity.getDing_talk_process_id());
        fileMap.put("userCode", "tbadmin");//审批人
        fileMap.put("ckeckResult", "R");
        fileMap.put("ckeckNote", "无");
        String jsonFileMap = JSONUtil.toJsonStr(fileMap);


        //请求
        HttpClient httpClient = new HttpClient();
        PostMethod httpPost = new PostMethod(serviceUrl + serviceName);
        httpPost.setRequestHeader("content-type", "application/json;charset=utf-8");
        for (Map.Entry<String, Object> entry : map.entrySet()) {
            httpPost.setRequestHeader(entry.getKey(), entry.getValue()
                    .toString());
        }

        RequestEntity entity = new StringRequestEntity(jsonFileMap, "application/json", "UTF-8");
        httpPost.setRequestEntity(entity);
        httpClient.executeMethod(httpPost);
        String responseBodyAsString = httpPost.getResponseBodyAsString();
        System.out.println(responseBodyAsString);

        //记录日志
        dingU8cBillEntity.setApprove("N");
        dingU8cBillEntity.setPush_status("N");
        dingU8cBillDao.callBackSaveOrUpdate(dingU8cBillEntity);
    }

    @Autowired
    private IArapDjzbDao arapDjzbDao;
    /**
     * 查询u8c付款单主表
     * @param u8cPkCorp
     * @param u8cBillCode
     * @return
     */
    public ArapDjzbEntity queryArapDjzbEntity(String u8cPkCorp, String u8cBillCode){
        ArapDjzbEntity arapDjzbEntity = new ArapDjzbEntity();
        arapDjzbEntity.setDr("0");
        arapDjzbEntity.setDwbm(u8cPkCorp);
        arapDjzbEntity.setDjbh(u8cBillCode);
        arapDjzbEntity.setDataSourceCode(ProfilesActiveConstant.LETS_DATE_SOURCE);
        List<ArapDjzbEntity> query = arapDjzbDao.query(arapDjzbEntity);
        if(query.size()==0){
            return null;
        }else {
            return query.get(0);
        }
    }

    /**
     * 获取钉钉应用token
     */
    public String getDingTalkToken() throws Exception {
        GetAccessTokenResponseBody tokenBody = DingTalkUtils.getTokenBody(APPKEY, AppSecret);
        return tokenBody.getAccessToken();
    }
    public String getMobileByUserId(String token, String userId) {
        OapiV2UserGetResponse user = DingTalkUtils.getUserById(token, userId);
        String mobile = user.getResult().getMobile();
        return mobile;
    }

}
